dnl pcap.h winds up in wierd places.

AC_DEFUN(AC_WF_CHECK_PCAP_HEADER_DIR,
          [AC_MSG_CHECKING(for pcap header directory)
		  ac_pcap_header_dir="none"
		  for dir in $prefix/include /include /usr/include /usr/local/include /include/pcap /usr/include/pcap /usr/local/include/pcap
		  do
		    if test -f "$dir/pcap.h" ; then
			  ac_pcap_header_dir=$dir
		    fi
	  done
		  
	  if test $ac_pcap_header_dir = "none" ; then
          AC_MSG_ERROR([
I cannot find the pcap header file anywhere. Perhaps you did not install 
the pcap package, or you did not install its development header files.], 1)
	  fi
	  
	  AC_MSG_RESULT($ac_pcap_header_dir)])

dnl on some broken systems /usr/local/lib is not in the linker's
dnl default path. so we'll sift about.

AC_DEFUN(AC_WF_CHECK_PCAP_LIB_DIR,
          [AC_MSG_CHECKING(for pcap library directory)
		  ac_pcap_lib_dir="none"
		  for dir in $prefix/lib /lib /usr/lib /usr/local/lib
		  do
		    if test -f "$dir/libpcap.a" ; then
			  ac_pcap_lib_dir=$dir
		    fi
	  done
		  
	  if test $ac_pcap_lib_dir = "none" ; then
                    AC_MSG_ERROR([
I cannot find the pcap library anywhere. Perhaps you did not install
the pcap package, or you did not install its development header files.], 1)
	  fi

	  AC_MSG_RESULT($ac_pcap_lib_dir)])

AC_DEFUN(AC_WF_TRY_PRI_MACROS,
[AC_MSG_CHECKING(checking for stdint conversion macros)
 AC_TRY_COMPILE([#include <stdio.h>
                 #include <stdlib.h>
                 #include <inttypes.h>
], [uint32_t t=1; printf("%"PRIu32, t)], 
                        [AC_MSG_RESULT(found)
                        have_primacros="yes"],
                        AC_MSG_RESULT(not found))])


AC_DEFUN(AC_WF_GET_PRI_MACROS,
[AC_TRY_RUN([
/* lack of C99 support caused me to do this. 
 * this code is probably hard to modify so don't
 * instead contact me for help.
 * 
 * I'm placing this in the public domain. Use it for
 * your project if you want.
 *
 * Thamer Alharbash tmh@whitefang.com */

#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <inttypes.h>
#include <errno.h>
#include <string.h>

#define TABLE_SIZE 4

/* substitutions as we know them. */

#define SUB_UNSIGNED_LONG "lu"
#define SUB_UNSIGNED_SHORT "hu"
#define SUB_LONG "li"
#define SUB_SHORT "hi"
#define SUB_INT   "d"
#define SUB_UNSIGNED_INT "du"
#define SUB_CHAR "d"                 /* same -- we're assuming int is always bigger than char */
#define SUB_UNSIGNED_CHAR "du"       /* which is an OK assumption. */




static void output_type_substitution(FILE *fp,
                                     char *sub_string,
                                     size_t size,
                                     char **sub_table,
                                     size_t *size_table)
{
    int i;

    for(i = 0;i < TABLE_SIZE; i++) {

        if(size_table[i] == size) {
            fprintf(fp, "#define %s \\"%s\\"\n", sub_string, sub_table[i]);
            break;
        }
    }

    if(i == TABLE_SIZE) {
        /* blah. we couldn't find a suitable type. Issue warning and use largest size.*/

        fprintf(stderr, "* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\n");
        fprintf(stderr, "* Warning! I couldn't find a suitable type that's sized for uint32_t. *\n");
        fprintf(stderr, "* I'm going to go and use the largest I can and hope things work!     *\n");
        fprintf(stderr, "* Please do report this bug to tmh@whitefang.com                      *\n");
        fprintf(stderr, "* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\n");

        fprintf(fp, "#define %s \\"%s\\"\n", sub_string, sub_table[0]);
    }

    return;
}
    
int main(int argc, char *argv[])
{
    FILE *fp;
    char *signed_sub_table[] = { SUB_LONG, SUB_SHORT, SUB_INT, SUB_CHAR };
    char *unsigned_sub_table[] = { SUB_UNSIGNED_LONG, SUB_UNSIGNED_SHORT, SUB_UNSIGNED_INT,
                                   SUB_UNSIGNED_CHAR };
    size_t signed_size_table[] = { sizeof(long), sizeof(short), sizeof(int), 
                                   sizeof(char) };
    size_t unsigned_size_table[] = { sizeof(unsigned long), sizeof(unsigned short), 
                                     sizeof(unsigned int), sizeof(unsigned char) };

    fp = fopen("primacros.h", "w");
    if(fp == NULL) {
        perror("fopen");
        exit(1);
    }

    fprintf(fp, "/* Automatically generated PRI macros. *\n");
    fprintf(fp, " * This is because you don't have them.*/\n");

    output_type_substitution(fp, "PRIu32", sizeof(uint32_t),
                             unsigned_sub_table, unsigned_size_table);

    output_type_substitution(fp, "PRIi32", sizeof(int32_t),
                             signed_sub_table, signed_size_table);


    output_type_substitution(fp, "PRIu16", sizeof(uint16_t),
                             unsigned_sub_table, unsigned_size_table);

    output_type_substitution(fp, "PRIi16", sizeof(int16_t),
                             signed_sub_table, signed_size_table);

    exit(0);
}
], [ac_have_primacros_h="1"], 
        AC_MSG_ERROR([I could not generate replacement C99 macros for stdint types! sorry I cannot carry on!]))])

AC_DEFUN(AC_WF_CHECK_KILL_SIGNAL_DETECT,
[AC_MSG_CHECKING(if kill(0, pid) can be used to detect a process)
AC_TRY_RUN([
/* Check the behaviour of kill(pid, 0).
 * We expect it to return 0 for an existing process that we can
 * signal, and < 0 otherwise.
 */

#include <stdlib.h> 
#include <stdio.h>

#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <unistd.h>

int main(int argc, char *argv[])
{
    pid_t child_pid;
    int kill_existing, kill_nonexisting;
    int wait_status;

    child_pid = fork();

    if (child_pid == -1) {
        perror("fork");
        exit(1);
    } else if (child_pid == 0) {
	(void)sleep(3600);
        _exit(0);
    }

    kill_existing = kill(child_pid, 0);

    (void)kill(child_pid, SIGTERM);
    (void)wait(&wait_status);

    kill_nonexisting = kill(child_pid, 0);

    if (kill_existing == 0 && kill_nonexisting < 0)
        exit(0); /* the behaviour we expect */
    else
        fprintf(stderr, "error: returned (%d,%d); expecting (0,-1)\n",
                kill_existing, kill_nonexisting);

    exit(1);
}
], [AC_DEFINE(KILL_SIGNAL_DETECT) AC_MSG_RESULT(yes)], [AC_MSG_RESULT(no)])])


AC_DEFUN(AC_WF_CHECK_BPF_IMMEDIATE,
[AC_MSG_CHECKING(for bpf immediate ioctl)
 AC_TRY_COMPILE([#include <stdio.h>
                 #include <stdlib.h>
                 #include <sys/types.h>
                 #include <sys/time.h>
                 #include <sys/ioctl.h>
                 #include <net/bpf.h>
                 #include <net/bpfdesc.h>
], [    ioctl(0, BIOCIMMEDIATE, NULL);], 
                        [AC_MSG_RESULT(found)
                        AC_DEFINE(HAVE_BPF_IMMEDIATE)],
                        AC_MSG_RESULT(not found))])
