/* $Header: /cvsroot/dhcp-agent/dhcp-agent/dhcp-arp-discovery.c,v 1.6 2002/06/22 01:47:57 actmodern Exp $
 * 
 * Copyright 2002 Thamer Alharbash <tmh@whitefang.com>
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 3. The names of the authors may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include <dhcp-agent.h>
#include <dhcp-util.h>
#include <dhcp-net.h>

/* Accept a list of arguments. Will contain rawnet first, and
 * then the ip address we're interested in. */

static int check_for_arp_reply(void *data)
{
    rawnet_t *net;
    unsigned char *addr, *received_addr;
    list_t *argument_list = data;

    net = argument_list->data;
    if(argument_list->next)
        addr = argument_list->next->data;
    else {
        warn_message("check arp reply: no address passed to function -- skipping");
        return 0;
    }

    received_addr = arp_get_sender_protocol_address(net->arp_p);

    if((net->type == RAWNET_ARP) &&
       (arp_get_hardware_type(net->arp_p) == ARP_HRD_ETH) &&
       (arp_get_protocol_type(net->arp_p) == ARP_PRO_IP) &&
       (arp_get_hardware_len(net->arp_p) == ETH_ADDR_LEN) &&
       (arp_get_protocol_len(net->arp_p) == IP_ADDR_LEN) &&
       (arp_get_op(net->arp_p) == ARP_OP_REPLY)) {

        /* now check if its the address we're looking for */
        if(!memcmp(addr, received_addr, IP_ADDR_LEN))
            return 1; /* yes! address match */
        else
            return 0; /* nope we got another response. */
    }

    return 0; /* not the right kind of packet. */
}

int arp_discover_hardware_address(rawnet_t *net, int retries, uint32_t address, eth_addr_t *mac_addr)
{
    list_t *arg_list = NULL;
    unsigned char *sender_addr;

    arg_list = add_to_list(arg_list, &address);
    arg_list = add_to_list(arg_list, net);

    build_arp_request(net, net->ip_addr, address, net->hw_addr); /* 0 for source ip, address to discover for dest,
                                                                  * our hardware address so we get a reply back. */
    while(retries--) {

        if(rawnet_packet_transact(net, arg_list, NULL, check_for_arp_reply))
            continue;
    
        /* Yummy! We got a valid reply! */
        
        sender_addr = arp_get_sender_hardware_address(net->arp_p);
        memcpy(mac_addr->data, sender_addr, ETH_ADDR_LEN);

        /* just free up pair. FIXME: this isn't good because we're 
         * breaking the list blackbox. we need a better way to do this. 
         * the datums don't need to be freed since we keep them on the stack. */

        xfree(arg_list->next);
        xfree(arg_list);

        return 0;
    }
    
    /* FIXME: ditto (see above) */
    xfree(arg_list->next);
    xfree(arg_list);

    /* no response, return error. */
    return 1;
}
