/* $Header: /cvsroot/dhcp-agent/dhcp-agent/dhcp-cache-entry.c,v 1.8 2002/06/17 13:23:07 actmodern Exp $
 * 
 * Copyright 2002 Thamer Alharbash <tmh@whitefang.com>
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 3. The names of the authors may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * Cache entry object:
 * 
 * We use this object to hold a name/value duplet.
 * 
 */

#include <dhcp-agent.h>
#include <dhcp-util.h>

/* constructors. */
cache_entry_t *make_cache_entry(const char *name, const char *value)
{
    cache_entry_t *cache;
    
    cache = xmalloc(sizeof(cache_entry_t));
    cache->name = strdup(name);
    cache->value = strdup(value);
    
    return cache;
}

void destroy_cache_entry(cache_entry_t *cache)
{
    xfree(cache->name);
    xfree(cache->value);
    xfree(cache);
    
    return;
}

/* use alongside purge_list or similar routines. */
static void destroy_cache_entry_in_list(void *data)
{
    destroy_cache_entry(data);
}

/* purge list of cache entries. */
void cache_entry_purge_list(list_t *cache_list)
{
    purge_list(cache_list, destroy_cache_entry_in_list);
    return;
}
