/* $Header: /cvsroot/dhcp-agent/dhcp-agent/dhcp-convert.h,v 1.2 2002/06/17 13:23:08 actmodern Exp $
 *
 * Copyright 2002 Thamer Alharbash
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 3. The names of the authors may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#ifndef DHCP_CONVERT_H
#define DHCP_CONVERT_H

/* conversion handler. */

typedef struct option_convert_handler {
    char* (*serialize)(const unsigned char *data, int len);
    void* (*serialize_to_internal)(const char *string);
    void* (*serialize_to_network)(const char *string, unsigned char *retlen);
    void  (*free_internal)(void *ptr);
} option_convert_handler;

/* global conversion handlers. */

extern option_convert_handler option_convert_handlers[];

/* conversion routines. */

/* byte routines. */

extern char *network_byte_to_string_byte(const unsigned char *data, int len);
extern void *string_byte_to_byte(const char *s);
extern void *string_byte_to_network(const char *s, unsigned char *ret_len);

/* address routines. */

extern char *network_addr_to_string(const unsigned char *data, int len);
extern char *network_addr_list_to_string(const unsigned char *data, int len);
extern char *addr_list_to_string(const unsigned char *data, int len);
extern void *string_to_addr_list(const char *string);
extern void *string_to_addr(const char *string);
extern void *string_addr_list_to_network(const char *string, unsigned char *len_ret);
extern void *string_addr_to_network(const char *string, unsigned char *len_ret);
extern char *network_addr_pair_list_to_string(const unsigned char *data, int len);
extern void *string_to_addr_pair_list(const char *s);
extern void *string_addr_pair_list_to_network(const char *s, unsigned char *retlen);

/* integer routines. */

extern char *network_uint32_to_string(const unsigned char *data, int len);
extern char *network_int32_to_string(const unsigned char *data, int len);
extern char *network_uint16_to_string(const unsigned char *data, int len);
extern char *network_int16_to_string(const unsigned char *data, int len);

extern char *uint32_to_string(const unsigned char *data, int len);
extern char *int32_to_string(const unsigned char *data, int len);
extern char *uint16_to_string(const unsigned char *data, int len);
extern char *int16_to_string(const unsigned char *data, int len);

extern void *string_to_uint32(const char *s);
extern void *string_to_int32(const char *s);
extern void *string_to_uint16(const char *s);
extern void *string_to_int16(const char *s);

extern void *string_int32_to_network(const char *s, unsigned char *ret_len);
extern void *string_uint32_to_network(const char *s, unsigned char *ret_len);
extern void *string_int16_to_network(const char *s, unsigned char *ret_len);
extern void *string_uint16_to_network(const char *s, unsigned char *ret_len);

extern char *network_uint16_list_to_string(const unsigned char *data, int len);
extern void *string_to_uint16_list(const char *s);
extern void *string_uint16_list_to_network(const char *s, unsigned char *retlen);

/* network to string */

extern char *validate_network_string_to_string(const unsigned char *data, int len);
extern void *strdup_wrap(const char *s);
extern void *strdup_wrap_net(const char *s, unsigned char *ret_len);

/* static network address conversion routine */

extern char *network_address_to_string_static(uint32_t addr);

#endif /* DHCP_CONVERT_H */
