/* $Header: /cvsroot/dhcp-agent/dhcp-agent/dhcp-exception.c,v 1.1 2002/06/22 18:11:25 actmodern Exp $
 *
 * Copyright 2002 Thamer Alharbash
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 3. The names of the authors may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * Poor man's C exceptions. This type of exception is not meant to be catchable.
 * Instead it's meant to hold error strings from functions as they progress up
 * the stack
 */

#include <dhcp-agent.h>
#include <dhcp-util.h>
#include <dhcp-exception.h>

static exception_t exceptions;

/* initialize global exception module. */
void init_exceptions(void)
{
    exceptions.cur_index = 0;
    return;
}

void push_exception(const char *module_name, const char *function_name,
                    char *fmt, ...)
{
    va_list ap;

    exceptions.module_name[exceptions.cur_index] = module_name;
    exceptions.function_name[exceptions.cur_index] = function_name;
    exceptions.exception_message[exceptions.cur_index] = create_stringbuffer();

    va_start(ap, fmt);
    stringbuffer_avprintf(exceptions.exception_message[exceptions.cur_index],
                          fmt, ap);
    va_end(ap);

    exceptions.cur_index++;

    if(exceptions.cur_index == MAX_EXCEPTION_STACK) {
        fatal_error("push_exception: ran out of exception stack space!");
        exit(1);
    }

}

/* returns unrolled full stack trace. 
 * destroys the strings as they're unrolled. */
stringbuffer *unroll_exceptions(void)
{
    int i;
    stringbuffer *unrolled_exception = create_stringbuffer();

    /* format is as follows. 
     * module_name:
     *   function_name: 
     *     msg msg msg msg msg msg msg
     *     msg msg msg msg msg msg msg
     *     msg msg msg msg msg msg msg
     */

    for(i = 0; i < exceptions.cur_index; i++) {

        /* append module name. */
        stringbuffer_aprintf(unrolled_exception, "%s:", exceptions.module_name[i]);
        /* append function name. */
        stringbuffer_aprintf(unrolled_exception, "  %s():\n", exceptions.function_name[i]);
        stringbuffer_aprintf_align(unrolled_exception, 4, 50, "%s\n",
                                   stringbuffer_getstring(exceptions.exception_message[i]));
        destroy_stringbuffer(exceptions.exception_message[i]);
    }

    exceptions.cur_index = 0;
                                                         
    return unrolled_exception;
}
