/* $Header: /cvsroot/dhcp-agent/dhcp-agent/dhcp-net.h,v 1.2 2002/06/23 03:29:13 actmodern Exp $
 *
 * Copyright 2002 Thamer Alharbash
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 3. The names of the authors may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#ifndef DHCP_NET_H
#define DHCP_NET_H

/* Raw net types for packets. */

#define RAWNET_UNKNOWN 0
#define RAWNET_ARP 1
#define RAWNET_ICMP 2
#define RAWNET_DHCP 3

/* Raw net return types when
 * receiving packets. */

#define RAWNET_OK 0
#define RAWNET_PCAP_ERROR -1
#define RAWNET_MALFORMED_PACKET -2
#define RAWNET_UNHANDLED -3
#define RAWNET_TIMEOUT -4

/* Net/Rawnet routines. */

extern rawnet_t *rawnet_create(char *device, char *filter, int sport, int dport, int promiscuous);
extern void rawnet_destroy(rawnet_t *net);
extern int rawnet_get_packet(rawnet_t *net, struct timeval *tm);
extern int rawnet_get_hw_addr(rawnet_t *net, eth_addr_t *addr);
extern int rawnet_get_real_hw_addr(rawnet_t *net, eth_addr_t *addr);
extern int rawnet_get_ip_addr(rawnet_t *net, ip_addr_t *addr);
extern char *rawnet_strerror(rawnet_t *net);
extern int rawnet_send_packet(rawnet_t *net);
extern void rawnet_use_fake_hw_addr(rawnet_t *raw, char *mac_string);
extern void reinitialize_rawnet(rawnet_t *net, int promiscuous);


extern int rawnet_up(rawnet_t *net);
extern void rawnet_down(rawnet_t *net);

extern int rawnet_packet_transact(rawnet_t *net, void *arg, 
                                  void (*update)(void *arg),
                                  int (*check)(void *arg));

extern int rawnet_interface_up(rawnet_t *net, ip_addr_t addr, ip_addr_t netmask, int mtu);
extern int rawnet_interface_down(rawnet_t *net);

extern void rawnet_dhcp_update(rawnet_t *net, time_t seconds);
extern int rawnet_is_dhcp_offer(rawnet_t *net);
extern int rawnet_is_valid(rawnet_t *net);

#endif
