/* $Header: /cvsroot/dhcp-agent/dhcp-agent/dhcp-rtt.c,v 1.7 2002/06/17 13:23:08 actmodern Exp $
 * 
 * Copyright 2002 Thamer Alharbash <tmh@whitefang.com>
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 3. The names of the authors may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#include <dhcp-agent.h>
#include <dhcp-util.h>
#include <dhcp-rtt.h>

static struct timeval make_timeval(time_t milliseconds)
{
    struct timeval tm;
    
    tm.tv_sec = MSECS_TO_SECS(milliseconds);
    tm.tv_usec = MSECS_TO_MSECS_REM(milliseconds);
    
    return tm;
}

rtt_t *rtt_create(void)
{
    rtt_t *rtt;
    time_t milliseconds;

    rtt = xmalloc(sizeof(rtt_t));
    milliseconds = MILLISECOND_TIMEOUT + rand()%MILLISECOND_RAND_TIMEOUT;
    rtt->timeout = make_timeval(milliseconds);

    return rtt;
}

struct timeval rtt_get_timeout(rtt_t *rtt)
{
    return rtt->timeout;
}

void rtt_timeout(rtt_t *rtt)
{    
    time_t milliseconds;
    int rand_value;
    
    milliseconds = SECS_TO_MSECS(rtt->timeout.tv_sec);
    milliseconds += rtt->timeout.tv_usec;
    milliseconds *= 2;
    
    rand_value = rand()%MILLISECOND_RAND_TIMEOUT;

    /* Add or subtract random small value. */
    if(rand()%10 > 5) /* we're safe adding. */
        milliseconds += rand()%MILLISECOND_RAND_TIMEOUT;
    else  {  /* we're not safe subtracting. don't go below 0. */

        if(rand_value < milliseconds)
            milliseconds -= rand_value;
        else
            milliseconds += rand_value;
    }
    
    rtt->timeout = make_timeval(milliseconds);
    return;
}

int rtt_can_retry(rtt_t *rtt)
{
    time_t milliseconds;
    
    milliseconds = SECS_TO_MSECS(rtt->timeout.tv_sec);
    milliseconds += rtt->timeout.tv_usec;

    if(milliseconds == 0) /* should never happen but let's check here
                           * for safe divide. */
        return 0;
    
    if(milliseconds >= (SECS_TO_MSECS(MAX_SECS_WAIT)))
        return 0;
    else
        return 1;
}

void rtt_destroy(rtt_t *rtt)
{
    xfree(rtt);
    return;
}
