/* $Header: /cvsroot/dhcp-agent/dhcp-agent/dhcp-sysconf.c,v 1.20 2002/06/22 01:47:57 actmodern Exp $
 * 
 * Copyright 2002 Thamer Alharbash <tmh@whitefang.com>
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 3. The names of the authors may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

/* Configure host internal structures. */

#include <dhcp-agent.h>
#include <dhcp-util.h>
#include <dhcp-files.h>
#include <dhcp-convert.h>
#include <dhcp-sysconf.h>
#include <dhcp-net.h>

/* Only use handlers here which can be done in any order.
 * Some special handlers are done in a particular order
 * before we run these routines.
 */

sysconf_handler sysconf_handlers[] = {
    { NULL, },                          /* 0 */  /* Pad -- don't handle here. */
    { NULL, },                          /* 1 */  /* Subnet mask -- don't handle here.*/
    { NULL, },                          /* 2 */  /* time offset. */
    { sysconf_routers, },               /* 3 */  /* routers. */
    { NULL, },                          /* 4 */  /* time server. */
    { NULL, },                          /* 5 */  /* name server (old). */
    { sysconf_domain_name_servers, },   /* 6 */  /* domain name servers. */
    { NULL, },                          /* 7 */  /* mit-lcs log server. */
    { NULL, },                          /* 8 */  /* cookie servers. */
    { NULL, },                          /* 9 */  /* lpr servers. */
    { NULL, },                          /* 10 */ /* impress server. */
    { NULL, },                          /* 11 */ /* resource location server. */
    { NULL, },                          /* 12 */ /* host name option. */
    { NULL, },                          /* 13 */ /* boot file size option. */
    { NULL, },                          /* 14 */ /* merit dump file. */
    { sysconf_domain_name, },           /* 15 */ /* domain name. */
    { NULL, },                          /* 16 */ /* swap server. */
    { NULL, },                          /* 17 */ /* root path. */
    { NULL, },                          /* 18 */ /* extension path */
    { NULL, },                          /* 19 */ /* ip forwarding. */
    { NULL, },                          /* 20 */ /* non local source routing. */
    { NULL, },                          /* 21 */ /* policy filter option. */
    { NULL, },                          /* 23 */ /* maximum dgram reassembly. */
    { NULL, },                          /* 24 */ /* mtu aging timeout. */
    { NULL, },                          /* 25 */ /* mtu plateau table. */
    { NULL, },                          /* 26 */ /* interface mtu */
    { NULL, },                          /* 27 */ /* subnets are local. */
    { NULL, },                          /* 28 */ /* broadcast address. */
    { NULL, },                          /* 29 */ /* perform mask discovery. */
    { NULL, },                          /* 30 */ /* mask supplier. */
    { NULL, },                          /* 31 */ /* perform router discovery. */
    { NULL, },                          /* 32 */ /* router soliciation address. */
    { NULL, },                          /* 33 */ /* static route. */
    { NULL, },                          /* 34 */ /* trailer encapsulation. */
    { NULL, },                          /* 35 */ /* arp cache timeout. */
    { NULL, },                          /* 36 */ /* ethernet encapsulation. */
    { NULL, },                          /* 37 */ /* tcp default ttl. */
    { NULL, },                          /* 38 */ /* tcp keepalive garbage option. */
    { NULL, },                          /* 39 */ /* network information service domain. */
    { NULL, },                          /* 40 */ /* network time protocol servers. */
    { NULL, },                          /* 41 */ /* vendor specific. */
    { NULL, },                          /* 42 */ /* netbios over TCP/IP name server. */
    { NULL, },                          /* 43 */ /* netbios over tcp/ip */
    { NULL, },                          /* 44 */
    { NULL, },                          /* 45 */
    { NULL, },                          /* 46 */
    { NULL, },                          /* 47 */
    { NULL, },                          /* 48 */
    { NULL, },                          /* 49 */
    { NULL, },                          /* 50 */
    { NULL, },                          /* 51 */
    { NULL, },                          /* 52 */
    { NULL, },                          /* 53 */
    { NULL, },                          /* 54 */
    { NULL, },                          /* 55 */
    { NULL, },                          /* 56 */
    { NULL, },                          /* 57 */
    { NULL, },                          /* 58 */
    { NULL, },                          /* 59 */
    { NULL, },                          /* 60 */
    { NULL, },                          /* 61 */
};

int sysconf_routers(void *value, dhcp_client_control_t *dc)
{
    struct route_entry r_entry;
    uint32_t *addr_val = NULL;
    route_t *rt;
    list_t *routers;
    list_t *average_latencies, *list_ptr;
    int lowest_latency;
    int *latency;

    memset(&r_entry.route_dst, 0, sizeof(r_entry.route_dst));
    memset(&r_entry.route_gw, 0, sizeof(r_entry.route_gw));
    
    routers = value;
    average_latencies = icmp_rtt_discovery(dc->rawnet, routers);

    /* Even if all routers fail to respond to the ICMP echo query,
     * we still want to use one of them.
     * (See RFC 1812, section 4.3.3.6.) */
    addr_val = average_latencies->data;
    lowest_latency = *(int *)average_latencies->next->data;

    /* Now pick best router and use it as default. */
    for(list_ptr = average_latencies;
        list_ptr != NULL;
        list_ptr = list_ptr->next->next) {

        latency = list_ptr->next->data;
        if(*latency != -1 && *latency < lowest_latency) {
            lowest_latency = *latency;
            addr_val = list_ptr->next->data;
        }
    }

    r_entry.route_dst.addr_type = ADDR_TYPE_IP;
    r_entry.route_dst.addr_bits = 0;
    r_entry.route_dst.addr_ip = 0;
    
    r_entry.route_gw.addr_type = ADDR_TYPE_IP;
    r_entry.route_gw.addr_bits = 0;
    memcpy(&r_entry.route_gw.addr_ip, addr_val, IP_ADDR_LEN);

    purge_list(average_latencies, NULL);

    rt = route_open();
    if(rt == NULL) {
        return -1;
    }

    if(route_add(rt, &r_entry) < 0)  {
        route_close(rt);
        return -1;
    }

    route_close(rt);
    return 0;
}

int sysconf_domain_name_servers(void *value, dhcp_client_control_t *dc)
{
    FILE *fp;
    list_t *servers;
    struct in_addr addr;

    fp = file_open_or_create_safe("resolv.conf", "a");
    if(fp == NULL)
        return -1;
    
    for(servers = value; servers; servers = servers->next)  {
        memcpy(&addr.s_addr, servers->data, IP_ADDR_LEN);
        fprintf(fp, "nameserver %s\n", inet_ntoa(addr));
    }
    
    fclose(fp);
    return 0;
}

int sysconf_domain_name(void *value, dhcp_client_control_t *dc)
{
    FILE *fp;
    char *dm;
    
    fp = file_open_or_create_safe("resolv.conf", "a");
    if(fp == NULL)
        return -1;
    
    dm = value;
    fprintf(fp, "search %s\n", dm);
    
    fclose(fp);
    return 0;
}

static void sysconf_cleanup_files(void)
{
    /* these files are in our work directory,
     * so we won't do anything dangerous to the system yet. */
    
    delete_file("resolv.conf");
    return;
}

static void sysconf_setup_interface(list_t *options, dhcp_client_control_t *dc)
{
    list_t *list_ptr;
    uint32_t addr, netmask;
    unsigned char have_addr = 0;
    unsigned char have_netmask = 0;
    int mtu = DEFAULT_MTU;
    cache_entry_t *ce;
    void *data;
    
    info_message("configuring interface: %s", dc->interface);

    for(list_ptr = options;list_ptr;list_ptr = list_ptr->next)  {

        ce = list_ptr->data;

        if(string_matches(dhcp_options_strings[TAG_DHCP_REQUEST_IP_ADDRESS], 
                          ce->name))  {

            data = option_convert_handlers[TAG_DHCP_REQUEST_IP_ADDRESS].serialize_to_internal(ce->value);
            memcpy(&addr, data, IP_ADDR_LEN);
            xfree(data);
            have_addr = 1;
            
            if(have_addr && have_netmask) /* if we're done finding both, break out. */
                break;

            continue;
        }

        if(string_matches(dhcp_options_strings[TAG_DHCP_SUBNET_MASK],
                          ce->name))  {

            data = option_convert_handlers[TAG_DHCP_SUBNET_MASK].serialize_to_internal(ce->value);
            memcpy(&netmask, data, IP_ADDR_LEN);
            xfree(data);
            have_netmask = 1;

            if(have_addr && have_netmask) /* if we're done finding both, break out. */
                break;

            continue;
        }
    }

    if(!have_addr && !have_netmask)  
        fatal_error("do not have enough settings for interface configuration!");
    if(rawnet_interface_up(dc->rawnet, addr, netmask, mtu))
        fatal_error("could not bring up interface: %s", dc->interface);

    return;
}

void do_sysconf(list_t *options, dhcp_client_control_t *dc)
{
    list_t *list_ptr;
    void *data;
    cache_entry_t *ce;
    unsigned char i;

    /* Clean up any config files we have. */

    sysconf_cleanup_files();

    /* Setup our interface it. */
    
    sysconf_setup_interface(options, dc);

    /* we need to setup the interface ourselves before
     * anything (route won't work otherwise.) */

    for(list_ptr = options;list_ptr;list_ptr = list_ptr->next)  {

        ce = list_ptr->data;
        
        for(i = 0;i < MAX_OPTIONS_HANDLED;i++)  {

            if(string_matches(dhcp_options_strings[i], ce->name))  {

                /* We have a match now check for sysconf and serialize_to_internal
                 * handler. */
                
                if(sysconf_handlers[i].sysconf_handler == NULL ||
                   option_convert_handlers[i].serialize_to_internal == NULL)
                    break;
            
                info_message("configuring for option %s", dhcp_options_strings[i]);

                data = option_convert_handlers[i].serialize_to_internal(ce->value);

                if(sysconf_handlers[i].sysconf_handler(data, dc))
                    error_message("error configuring for option %s: %s", dhcp_options_strings[i],
                                  strerror(errno));

                option_convert_handlers[i].free_internal(data);
            }
        }
    }
    
    /* we don't need to worry about race conditions.
     * /etc should NOT be world writable! */
    
    /* FIXME: perhaps we should have file handlers in the sysconf array? */
    
    if(file_exists("resolv.conf"))  {
        move_file("resolv.conf", "/etc/resolv.conf");
        make_file_public_read("/etc/resolv.conf");
    }

    return;
}
