/* $Header: /cvsroot/dhcp-agent/dhcp-agent/dhcp-util.h,v 1.7 2002/06/22 18:11:25 actmodern Exp $
 *
 * Copyright 2002 Thamer Alharbash
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 3. The names of the authors may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#ifndef DHCP_UTIL_H
#define DHCP_UTIL_H

#include <dhcp-stringbuffer.h>

/* Utility functions. */

/* in case strdup is not available. */

# ifndef HAVE_STRDUP
extern char *strdup(char *s);
# endif /* HAVE_STRDUP */

extern void error_message(char *fmt, ...);
extern void fatal_error(char *fmt, ...);
extern void warn_message(char *fmt, ...);
extern void debug_message(char *fmt, ...);
extern void fatal_error_exception(stringbuffer *exception_string);

extern int get_verbosity_level(void);
extern int set_verbosity_level(int verbosity_level_to_set);

extern void *xmalloc(size_t size);
extern void *xcalloc(size_t size);
extern void xfree(void *p);
extern int check_for_interrupts(void);
extern void suspend_for_interrupts(void);
extern void block_interrupts(void);
extern void add_interrupt_handler(int sigtype, void (*sighandler)(int));
extern void remove_interrupt(int sig);
extern int had_alarm(void);
extern void set_alarm(struct timeval alarm_time);
extern void remove_alarm(void);
extern void info_message(char *fmt, ...);
extern char *splice_string(const char *s1, const char *s2);
extern char *splice_many_strings(int num, char *s, ...);
extern void trim_string(char *s);
extern int string_matches(const char *s1, const char *s2);
extern int hex_string_to_value(char *string, unsigned char *dst);
extern int is_string(const char *string, int len);
extern uint16_t get_random_uint16(void);
extern uint32_t get_random_uint32(void);
extern struct timeval timeval_diff(struct timeval begin, struct timeval end);
extern const char *getprogname(void);
extern void track_sigio(void);
extern int had_io(void);
extern int resolv(char *address, uint32_t *addr);
extern int port_for_service(const char *serv, const char *proto);

#endif /* DHCP_UTIL_H */
