/* $Header: /cvsroot/dhcp-agent/dhcp-agent/dhcp-varfile.c,v 1.3 2002/06/22 18:11:25 actmodern Exp $
 * Copyright 2002 Thamer Alharbash <tmh@whitefang.com> 
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 3. The names of the authors may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * varfiles -- variable files. All the files which we read are varfiles.
 *
 */

#include <dhcp-agent.h>
#include <dhcp-util.h>
#include <dhcp-files.h>
#include <dhcp-parser.h>
#include <dhcp-varfile.h>

const char *varfile_get_name(varfile_t *varfile)
{
    return stringbuffer_getstring(varfile->var_name);
}

const char *varfile_get_val(varfile_t *varfile)
{
    return stringbuffer_getstring(varfile->var_val);
}

int varfile_get_lineno(varfile_t *varfile)
{
    return parser_get_line_no(varfile->parser);
}

varfile_t *create_varfile(const char *filename, uint8_t mode)
{
    varfile_t *varfile;

    varfile = xmalloc(sizeof(varfile_t));
    varfile->filename  = strdup(filename);
    varfile->var_name = create_stringbuffer();
    varfile->var_val = create_stringbuffer();
    varfile->parser = create_parser(filename);
    varfile->mode = mode;
    return varfile;
}

void destroy_varfile(varfile_t *varfile)
{
    destroy_parser(varfile->parser);
    destroy_stringbuffer(varfile->var_name);
    destroy_stringbuffer(varfile->var_val);
    xfree(varfile->filename);
    xfree(varfile);

    return;
}

const char *varfile_get_filename(varfile_t *varfile)
{
    return varfile->filename;
}


int varfile_get_next_varnames(varfile_t *varfile)
{
    atom_t atom;

    while(1) {

        atom = get_next_atom(varfile->parser);

        if(atom == PARSER_ATOM_END_OF_STATEMENT) /* skip empty lines/comments. */ 
            continue;

        if(atom == PARSER_ATOM_EOF)
            return VARFILE_EOF;

        if(atom != PARSER_ATOM_STRING)
            return VARFILE_PARSE_ERROR;

        /* copy string out into varname */
        stringbuffer_set(varfile->var_name, parser_get_data(varfile->parser));
        atom = get_next_atom(varfile->parser);

        /* if we don't get an end of statement or eof here it's a parse error. */
        if(atom != PARSER_ATOM_END_OF_STATEMENT &&
           atom != PARSER_ATOM_EOF)
            return VARFILE_PARSE_ERROR;

        return VARFILE_SUCCESS;
    }
}

int varfile_get_next_varval(varfile_t *varfile)
{
    atom_t atom;

    /* get the name first. */
    while(1) {

        atom = get_next_atom(varfile->parser);

        if(atom == PARSER_ATOM_END_OF_STATEMENT) /* skip empty lines/comments. */ 
            continue;

        if(atom == PARSER_ATOM_EOF)
            return VARFILE_EOF;

        if(atom != PARSER_ATOM_STRING)
            return VARFILE_PARSE_ERROR;

        /* copy string out into varname */
        stringbuffer_set(varfile->var_name, parser_get_data(varfile->parser));
        break;
    }

    /* the rest should be ASSIGNMENT STRING END_OF_STATEMENT; */

    atom = get_next_atom(varfile->parser);

    if(atom != PARSER_ATOM_ASSIGNMENT)
        return VARFILE_PARSE_ERROR;

    atom = get_next_atom(varfile->parser);

    if(atom != PARSER_ATOM_STRING)
        return VARFILE_PARSE_ERROR;

    /* copy string out into var val */
    stringbuffer_set(varfile->var_val, parser_get_data(varfile->parser));

    /* now we need a clean end of statement. */

    atom = get_next_atom(varfile->parser);

    if(atom != PARSER_ATOM_END_OF_STATEMENT &&
       atom != PARSER_ATOM_EOF)
        return VARFILE_PARSE_ERROR;

    return VARFILE_SUCCESS;
}

/* Get the next var statement from file. */
int varfile_get_next_var(varfile_t *varfile)
{
    switch(varfile->mode) {

      case VARFILE_VARNAMES_MODE:
          return(varfile_get_next_varnames(varfile));

      case VARFILE_VARVAL_MODE:
          return(varfile_get_next_varval(varfile));

    default:
        fatal_error("varfile: file_get_next_var: encountered a fatal bug -- please report this.");
    }

    /* should never get here. */
    return 0;
}
