/* $Header: /cvsroot/dhcp-agent/dhcp-agent/dhcp-varfile.h,v 1.2 2002/06/17 13:23:08 actmodern Exp $
 * Copyright 2002 Thamer Alharbash <tmh@whitefang.com> 
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 3. The names of the authors may not be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * Parser for varfiles.
 *
 */

#ifndef DHCP_VARFILE_H
#define DHCP_VARFILE_H

typedef struct  {
    char *filename;
    parser_t *parser;
    stringbuffer *var_name;
    stringbuffer *var_val;
    uint8_t mode;
} varfile_t;

/* open/close a varfile. */
extern varfile_t *create_varfile(const char *filename, uint8_t mode);
extern void destroy_varfile(varfile_t *varfile);

/* get variables and variable names. */
extern const char *varfile_get_name(varfile_t *varfile);
extern const char *varfile_get_val(varfile_t *varfile);
extern int varfile_get_lineno(varfile_t *varfile);
extern const char *varfile_get_filename(varfile_t *varfile);

/* get the next var */
int varfile_get_next_var(varfile_t *varfile);

/* modes for varfiles. */
#define VARFILE_VARNAMES_MODE 0 /* just names. */
#define VARFILE_VARVAL_MODE   1 /* names and values. */

/* return values for get_next_var */
#define VARFILE_PARSE_ERROR -1  /* parse error. */
#define VARFILE_EOF          0  /* end of file. */
#define VARFILE_SUCCESS      1  /* success */

#endif /* DHCP_VARFILE_H */
